import TRANSLATE_CS from "./cs/translations.json";
import TRANSLATE_EN from "./en/translations.json";

export const translations = {
    cs: { translation: TRANSLATE_CS },
    en: { translation: TRANSLATE_EN },
}
