{{cookiecutter.app_package|upper}}_RESOURCE = "{{cookiecutter.app_package}}.resources.resource:{{cookiecutter.resource}}"
{{cookiecutter.app_package|upper}}_RESOURCE_CONFIG = "{{cookiecutter.app_package}}.resources.config:{{cookiecutter.resource_config}}"
