#!/usr/bin/env sh

i18next-conv -l cs -s ./messages/cs/translations.json -t ./messages/cs/messages.po
i18next-conv -l en -s ./messages/en/translations.json -t ./messages/en/messages.po
i18next-conv -l el -s ./messages/el/translations.json -t ./messages/el/messages.po
i18next-conv -l de -s ./messages/de/translations.json -t ./messages/de/messages.po
